@@ -28,7 +28,7 @@
 //   it is available (slist is part of the SGI STL but not the C++
 //   standard) for better performance.
 
-namespace acommon {
+namespace aspell {
 
   template <typename T>
   class BasicList {
@@ -61,8 +61,6 @@ namespace acommon {
       data_.splice(data_.begin(),other.data_,cur);
       //data_.splice_after(data_.begin(), prev);
     }
-    template<class Pred>
-    void merge(BasicList & other, Pred pr) {data_.merge(other.data_, pr);}
   };
 
 }
