@@ -7,10 +7,12 @@
 #ifndef ASPELL_STRTONUM__HPP
 #define ASPELL_STRTONUM__HPP
 
-namespace acommon {
-
-  double strtod_c(const char * nptr, char ** endptr);
-  long strtoi_c(const char * npter,char ** endptr);
+namespace aspell {
+   
+  // Local independent numeric conversion.  It is OK if
+  //   nptr == *endptr
+  double strtod_c(const char * nptr, const char ** endptr);
+  long strtoi_c(const char * npter, const char ** endptr);
 
 }
 
