@@ -45,7 +45,7 @@
 #include <cstring>
 #include <cstdio>
 
-#include "iostream.hpp"
+//#include "iostream.hpp"
 
 #include "affix.hpp"
 #include "errors.hpp"
@@ -57,9 +57,9 @@
 #include "lsort.hpp"
 #include "hash-t.hpp"
 
-using namespace std;
+#include "gettext.h"
 
-namespace aspeller {
+namespace aspell { namespace sp {
 
 typedef unsigned char byte;
 static char EMPTY[1] = {0};
@@ -101,7 +101,7 @@ struct PfxEntry : public AffEntry
   PfxEntry() {}
 
   bool check(const LookupInfo &, const AffixMgr * pmyMgr,
-             ParmString, CheckInfo &, GuessInfo *) const;
+             ParmString, IntrCheckInfo &, GuessInfo *, bool cross = true) const;
 
   inline bool          allow_cross() const { return ((xpflg & XPRODUCT) != 0); }
   inline byte flag() const { return achar;  }
@@ -123,7 +123,7 @@ struct SfxEntry : public AffEntry
 
   SfxEntry() {}
 
-  bool check(const LookupInfo &, ParmString, CheckInfo &, GuessInfo *,
+  bool check(const LookupInfo &, ParmString, IntrCheckInfo &, GuessInfo *,
              int optflags, AffEntry * ppfx);
 
   inline bool          allow_cross() const { return ((xpflg & XPRODUCT) != 0); }
@@ -256,12 +256,12 @@ PosibErr<void> AffixMgr::setup(ParmString affpath, Conv & iconv)
   return parse_file(affpath, iconv);
 }
 
-AffixMgr::AffixMgr(const Language * l) 
+AffixMgr::AffixMgr(const LangImpl * l) 
   : lang(l), data_buf(1024*16) {}
 
 AffixMgr::~AffixMgr() {}
 
-static inline void MAX(int & lhs, int rhs) 
+static inline void max_(int & lhs, int rhs) 
 {
   if (lhs < rhs) lhs = rhs;
 }
@@ -369,8 +369,8 @@ PosibErr<void> AffixMgr::parse_file(const char * affpath, Conv & iconv)
         // key is strip 
         if (dp.key != "0") {
           ParmString s0(iconv(dp.key));
-          MAX(max_strip_, s0.size());
-          MAX(max_strip_f[(byte)achar], s0.size());
+          max_(max_strip_, s0.size());
+          max_(max_strip_f[(byte)achar], s0.size());
           nptr->strip = data_buf.dup(s0);
           nptr->stripl = s0.size();
         } else {
@@ -755,7 +755,7 @@ static void encodeit(CondsLookup & l, ObjStack & buf,
 
 // check word for prefixes
 bool AffixMgr::prefix_check (const LookupInfo & linf, ParmString word, 
-                             CheckInfo & ci, GuessInfo * gi) const
+                             IntrCheckInfo & ci, GuessInfo * gi, bool cross) const
 {
  
   // first handle the special case of 0 length prefixes
@@ -771,7 +771,7 @@ bool AffixMgr::prefix_check (const LookupInfo & linf, ParmString word,
 
   while (pptr) {
     if (isSubset(pptr->key(),word)) {
-      if (pptr->check(linf,this,word,ci,gi)) return true;
+      if (pptr->check(linf,this,word,ci,gi,cross)) return true;
       pptr = pptr->next_eq;
     } else {
       pptr = pptr->next_ne;
@@ -784,7 +784,7 @@ bool AffixMgr::prefix_check (const LookupInfo & linf, ParmString word,
 
 // check word for suffixes
 bool AffixMgr::suffix_check (const LookupInfo & linf, ParmString word, 
-                             CheckInfo & ci, GuessInfo * gi,
+                             IntrCheckInfo & ci, GuessInfo * gi,
                              int sfxopts, AffEntry * ppfx) const
 {
 
@@ -813,7 +813,7 @@ bool AffixMgr::suffix_check (const LookupInfo & linf, ParmString word,
 
 // check if word with affixes is correctly spelled
 bool AffixMgr::affix_check(const LookupInfo & linf, ParmString word, 
-                           CheckInfo & ci, GuessInfo * gi) const
+                           IntrCheckInfo & ci, GuessInfo * gi) const
 {
   // Deal With Case in a semi-intelligent manner
   CasePattern cp = lang->LangImpl::case_pattern(word);
@@ -838,18 +838,27 @@ bool AffixMgr::affix_check(const LookupInfo & linf, ParmString word,
   if (prefix_check(linf, pword, ci, gi)) return true;
 
   // if still not found check all suffixes
-  return suffix_check(linf, sword, ci, gi, 0, NULL);
+  if (suffix_check(linf, sword, ci, gi, 0, NULL)) return true;
+
+  // if still not found check again but with the lower case version
+  // which can make a difference if the entire word matches the cond
+  // string
+  if (cp == FirstUpper) {
+    return suffix_check(linf, pword, ci, gi, 0, NULL);
+  } else {
+    return false;
+  }
 }
 
-void AffixMgr::munch(ParmString word, GuessInfo * gi) const
+void AffixMgr::munch(ParmString word, GuessInfo * gi, bool cross) const
 {
   LookupInfo li(0, LookupInfo::AlwaysTrue);
-  CheckInfo ci;
+  IntrCheckInfo ci;
   gi->reset();
   CasePattern cp = lang->LangImpl::case_pattern(word);
   if (cp == AllUpper) return;
   if (cp != FirstUpper)
-    prefix_check(li, word, ci, gi);
+    prefix_check(li, word, ci, gi, cross);
   suffix_check(li, word, ci, gi, 0, NULL);
 }
 
@@ -868,8 +877,10 @@ WordAff * AffixMgr::expand(ParmString word, ParmString aff,
   WordAff * cur = head;
   cur->word = buf.dup(word);
   cur->aff  = suf;
-
-  for (const byte * c = (const byte *)aff.str(), * end = c + aff.size();
+  {
+    const byte * c = (const byte *)aff.str();
+    const byte * end = c + aff.size();
+    for (;
        c != end; 
        ++c) 
   {
@@ -885,7 +896,7 @@ WordAff * AffixMgr::expand(ParmString word, ParmString aff,
       cur->aff = p->allow_cross() ? csuf : empty;
     }
   }
-
+  }
   *suf_e = 0;
   *csuf_e = 0;
   cur->next = 0;
@@ -895,14 +906,14 @@ WordAff * AffixMgr::expand(ParmString word, ParmString aff,
   WordAff * * end = &cur->next;
   WordAff * * very_end = end;
   size_t nsuf_s = suf_e - suf + 1;
-
+  {
   for (WordAff * * cur = &head; cur != end; cur = &(*cur)->next) {
     if ((int)(*cur)->word.size - max_strip_ >= limit) continue;
     byte * nsuf = (byte *)buf.alloc(nsuf_s);
     expand_suffix((*cur)->word, (*cur)->aff, buf, limit, nsuf, &very_end, word);
     (*cur)->aff = nsuf;
   }
-
+  }
   return head;
 }
 
@@ -943,17 +954,18 @@ WordAff * AffixMgr::expand_suffix(ParmString word, const byte * aff,
 CheckAffixRes AffixMgr::check_affix(ParmString word, char aff) const
 {
   CheckAffixRes res = InvalidAffix;
-  
+  {  
   for (PfxEntry * p = pFlag[(unsigned char)aff]; p; p = p->flag_next) {
     res = InapplicableAffix;
     if (p->applicable(word)) return ValidAffix;
   }
-
+  }
+  {
   for (SfxEntry * p = sFlag[(unsigned char)aff]; p; p = p->flag_next) {
     if (res == InvalidAffix) res = InapplicableAffix;
     if (p->applicable(word)) return ValidAffix;
   }
-
+  }
   return res;
 }
 
@@ -996,7 +1008,7 @@ int LookupInfo::lookup (ParmString word, const SensitiveCompare * c,
     g = gi->dup(word);
   }
   if (gi && g) {
-    CheckInfo * ci = gi->add();
+    IntrCheckInfo * ci = gi->add();
     ci->word = g;
     return -1;
   }
@@ -1049,13 +1061,13 @@ SimpleString PfxEntry::add(SimpleString word, ObjStack & buf) const
 // check if this prefix entry matches 
 bool PfxEntry::check(const LookupInfo & linf, const AffixMgr * pmyMgr,
                      ParmString word,
-                     CheckInfo & ci, GuessInfo * gi) const
+                     IntrCheckInfo & ci, GuessInfo * gi, bool cross) const
 {
   unsigned int		cond;	// condition number being examined
-  int	                tmpl;   // length of tmpword
+  unsigned              tmpl;   // length of tmpword
   WordEntry             wordinfo;     // hash entry of root word or NULL
   byte *	cp;		
-  VARARRAYM(char, tmpword, word.size()+1, MAXWORDLEN+1);
+  VARARRAYM(char, tmpword, word.size()+stripl+1, MAXWORDLEN+1);
 
   // on entry prefix is 0 length or already matches the beginning of the word.
   // So if the remaining root word has positive length
@@ -1086,8 +1098,8 @@ bool PfxEntry::check(const LookupInfo & linf, const AffixMgr * pmyMgr,
     // root word in the dictionary
 
     if (cond >= conds->num) {
-      CheckInfo * lci = 0;
-      CheckInfo * guess = 0;
+      IntrCheckInfo * lci = 0;
+      IntrCheckInfo * guess = 0;
       tmpl += stripl;
 
       int res = linf.lookup(tmpword, &linf.sp->s_cmp_end, achar, wordinfo, gi);
@@ -1111,7 +1123,7 @@ bool PfxEntry::check(const LookupInfo & linf, const AffixMgr * pmyMgr,
       if (gi)
         lci = gi->head;
       
-      if (xpflg & XPRODUCT) {
+      if (cross && xpflg & XPRODUCT) {
         if (pmyMgr->suffix_check(linf, ParmString(tmpword, tmpl), 
                                  ci, gi,
                                  XPRODUCT, (AffEntry *)this)) {
@@ -1119,10 +1131,10 @@ bool PfxEntry::check(const LookupInfo & linf, const AffixMgr * pmyMgr,
           
         } else if (gi) {
           
-          CheckInfo * stop = lci;
+          IntrCheckInfo * stop = lci;
           for (lci = gi->head; 
                lci != stop; 
-               lci = const_cast<CheckInfo *>(lci->next)) 
+               lci = const_cast<IntrCheckInfo *>(lci->next)) 
           {
             lci->pre_flag = achar;
             lci->pre_strip_len = stripl;
@@ -1195,14 +1207,14 @@ SimpleString SfxEntry::add(SimpleString word, ObjStack & buf,
 
 // see if this suffix is present in the word 
 bool SfxEntry::check(const LookupInfo & linf, ParmString word,
-                     CheckInfo & ci, GuessInfo * gi,
+                     IntrCheckInfo & ci, GuessInfo * gi,
                      int optflags, AffEntry* ppfx)
 {
-  int	                tmpl;		 // length of tmpword 
+  unsigned              tmpl;		 // length of tmpword 
   int			cond;		 // condition beng examined
   WordEntry             wordinfo;        // hash entry pointer
   byte *	cp;
-  VARARRAYM(char, tmpword, word.size()+1, MAXWORDLEN+1);
+  VARARRAYM(char, tmpword, word.size()+stripl+1, MAXWORDLEN+1);
   PfxEntry* ep = (PfxEntry *) ppfx;
 
   // if this suffix is being cross checked with a prefix
@@ -1245,10 +1257,10 @@ bool SfxEntry::check(const LookupInfo & linf, ParmString word,
     // root word in the dictionary
 
     if (cond < 0) {
-      CheckInfo * lci = 0;
+      IntrCheckInfo * lci = 0;
       tmpl += stripl;
       const SensitiveCompare * cmp = 
-        optflags & XPRODUCT ? &linf.sp->s_cmp_middle : &linf.sp->s_cmp_end;
+        optflags & XPRODUCT ? &linf.sp->s_cmp_middle : &linf.sp->s_cmp_begin;
       int res = linf.lookup(tmpword, cmp, achar, wordinfo, gi);
       if (res == 1
           && ((optflags & XPRODUCT) == 0 || TESTAFF(wordinfo.aff, ep->achar)))
@@ -1283,7 +1295,7 @@ bool SfxEntry::check(const LookupInfo & linf, ParmString word,
 
 PosibErr<AffixMgr *> new_affix_mgr(ParmString name, 
                                    Conv & iconv,
-                                   const Language * lang)
+                                   const LangImpl * lang)
 {
   if (name == "none")
     return 0;
@@ -1303,7 +1315,7 @@ PosibErr<AffixMgr *> new_affix_mgr(ParmString name,
     return affix;
   }
 }
-}
+} }
 
 /**************************************************************************
 
