@@ -6,19 +6,19 @@
 
 #include "config.hpp"
 #include "convert.hpp"
+#include "file_util.hpp"
+#include "fstream.hpp"
 #include "data.hpp"
 #include "data_id.hpp"
 #include "errors.hpp"
-#include "file_util.hpp"
-#include "fstream.hpp"
-#include "language.hpp"
+#include "lang_impl.hpp"
 #include "speller_impl.hpp"
-#include "cache-t.hpp"
+#include "cache.hpp"
 #include "vararray.hpp"
 
-#include "iostream.hpp"
+#include "gettext.h"
 
-namespace aspeller {
+namespace aspell { namespace sp {
 
   GlobalCache<Dictionary> dict_cache("dictionary");
 
@@ -58,7 +58,7 @@ namespace aspeller {
     }
   }
 
-  PosibErr<void> Dictionary::attach(const Language &l) {
+  PosibErr<void> Dictionary::attach(const LangImpl &l) {
     if (lang_ && strcmp(l.name(),lang_->name()) != 0)
       return make_err(mismatched_language, lang_->name(), l.name());
     if (!lang_) lang_.copy(&l);
@@ -93,7 +93,7 @@ namespace aspeller {
   PosibErr<void> Dictionary::set_check_lang (ParmString l, Config & config)
   {
     if (lang_ == 0) {
-      PosibErr<Language *> res = new_language(config, l);
+      PosibErr<LangImpl *> res = new_lang_impl(config, l);
       if (res.has_err()) return res;
       lang_.reset(res.data);
       lang_->set_lang_defaults(config);
@@ -154,27 +154,32 @@ namespace aspeller {
   // BasicDict
   //
 
-//   class BasicDictEnumeration : public StringEnumeration 
-//   {
-//     BasicDict::Emul real_;
-//   public:
-//     BasicDictEnumeration(BasicDict::VirEmul * r) : real_(r) {}
-
-//     bool at_end() const {
-//       return real_.at_end();
-//     }
-//     const char * next() {
-//       return real_.next().word; // FIXME: It's not this simple
-//     }
-//     StringEnumeration * clone() const {
-//       return new BasicDictEnumeration(*this);
-//     }
-//     void assign(const StringEnumeration * other) {
-//       *this = *static_cast<const BasicDictEnumeration *>(other);
-//     }
-//   };
-
-
+  class DictStringEnumeration : public StringEnumeration 
+  {
+    ClonePtr<Dict::Enum> real_;
+    const LangImpl  * lang_;
+    String temp_str;
+  public:
+    DictStringEnumeration(Dict::Enum * r, const LangImpl  * l) 
+      : real_(r), lang_(l) {}
+    
+    bool at_end() const {
+      return real_->at_end();
+    }
+    const char * next() {
+      temp_str.clear();
+      WordEntry * w =  real_->next(); 
+      if (!w) return 0;
+      w->write(temp_str, *lang_);
+      return temp_str.str();
+    }
+    StringEnumeration * clone() const {
+      return new DictStringEnumeration(*this);
+    }
+    void assign(const StringEnumeration * other) {
+      *this = *static_cast<const DictStringEnumeration *>(other);
+    }
+  };
 
   PosibErr<void> Dictionary::add_repl(ParmString mis, ParmString cor) 
   {
@@ -203,8 +208,7 @@ namespace aspeller {
   //
 
   PosibErr<void> Dictionary::load(ParmString, Config &, 
-                                  LocalDictList *, 
-                                  SpellerImpl *, const LocalDictInfo *) 
+                                  DictList *, SpellerImpl *) 
   {
     return make_err(unimplemented_method, "load", class_name);
   }
@@ -236,7 +240,9 @@ namespace aspeller {
 
   StringEnumeration * Dictionary::elements() const
   {
-    return 0;
+    Enum * e = detailed_elements();
+    if (!e) return 0;
+    return new DictStringEnumeration(e, lang());
   }
   
   Dict::Enum * Dictionary::detailed_elements() const
@@ -255,8 +261,8 @@ namespace aspeller {
     return false;
   }
   
-  bool Dictionary::lookup (ParmString word, WordEntry &,
-                           const SensitiveCompare &) const
+  bool Dictionary::lookup (ParmString word, const SensitiveCompare *,
+                           WordEntry &) const
   {
     return false;
   }
@@ -311,39 +317,37 @@ namespace aspeller {
     return make_err(unimplemented_method, "remove_repl", class_name);
   }
   
-  Enumeration<const LocalDict *> * Dictionary::dictionaries() const 
+  DictsEnumeration * Dictionary::dictionaries() const 
   {
     return 0;
   }
 
 #define write_conv(s) do { \
     if (!c) {o << s;} \
-    else {ParmString ss(s); buf.clear(); c->convert(ss.str(), ss.size(), buf); o.write(buf.data(), buf.size()-1);} \
+    else {ParmString ss(s); buf.clear(); c->convert(ss.str(), ss.size(), buf); o.write(buf.data(), buf.size());} \
   } while (false)
 
   OStream & WordEntry::write (OStream & o,
-                              const Language & l,
+                              const LangImpl & l,
                               Convert * c) const
   {
-    String w;
     CharVector buf;
     write_conv(word);
     if (aff && *aff) {
-      o << '/';
+      o << '/'; // FIXME: This isn't write for UCS-2,4
       write_conv(aff);
     }
     return o;
   }
 
-  PosibErr<void> add_data_set(ParmString fn,
-                              Config & config,
-                              LocalDict & res,
-                              LocalDictList * new_dicts,
-                              SpellerImpl * speller,
-                              const LocalDictInfo * local_info,
-                              ParmString dir,
-                              DataType allowed)
+  PosibErr<Dict *> add_data_set(ParmString fn,
+                                Config & config,
+                                DictList * new_dicts,
+                                SpellerImpl * speller,
+                                ParmString dir,
+                                DataType allowed)
   {
+    Dict * res = 0;
     static const char * suffix_list[] = {"", ".multi", ".alias", 
 					 ".spcl", ".special",
 					 ".pws", ".prepl"};
@@ -387,7 +391,7 @@ namespace aspeller {
 
     } else {
       
-      char head[32];
+      char head[32] = {0};
       in.read(head, 32);
       if      (strncmp(head, "aspell default speller rowl", 27) ==0)
 	actual_type = DT_ReadOnly;
@@ -397,30 +401,29 @@ namespace aspeller {
 	actual_type = DT_Writable;
       else
 	return make_err(bad_file_format, true_file_name);
-
     }
     
     if (actual_type & ~allowed)
       return make_err(bad_file_format, true_file_name
 		      , _("is not one of the allowed types"));
 
-    Dict::Id id(0,Dict::FileName(true_file_name));
+    Dict::FileName id_fn(true_file_name);
+    Dict::Id id(0,id_fn);
 
     if (speller != 0) {
-      const LocalDict * d = speller->locate(id);
+      const SpellerDict * d = speller->locate(id);
       if (d != 0) {
-        res = *d;
-        return no_err;
+        return d->dict;
       }
     }
 
-    res.dict = 0;
+    res = 0;
 
     if (actual_type == DT_ReadOnly) { // try to get it from the cache
-      res.dict = dict_cache.find(id);
+      res = dict_cache.find(id);
     }
 
-    if (!res.dict) {
+    if (!res) {
 
       StackPtr<Dict> w;
       switch (actual_type) {
@@ -440,45 +443,24 @@ namespace aspeller {
         abort();
       }
 
-      RET_ON_ERR(w->load(true_file_name, config, new_dicts, speller, local_info));
+      RET_ON_ERR(w->load(true_file_name, config, new_dicts, speller));
 
       if (actual_type == DT_ReadOnly)
         dict_cache.add(w);
       
-      res.dict = w.release();
+      res = w.release();
 
     } else {
       
-      res.dict->copy();
+      res->copy();
       
     }
 
-    if (local_info) {
-      res.set(*local_info);
-      res.set_language(res.dict->lang());
-    } else
-      res.set(res.dict->lang(), config);
     if (new_dicts)
       new_dicts->add(res);
     
-    return no_err;
+    return res;
   }
 
-  //
-  // LocalDictInfo
-  //
-  
-  void LocalDictInfo::set_language(const Language * l)
-  {
-    compare.lang = l;
-  }
-
-  void LocalDictInfo::set(const Language * l, const Config & c)
-  {
-    compare.lang = l;
-    compare.case_insensitive = c.retrieve_bool("ignore-case");
-    compare.ignore_accents   = c.retrieve_bool("ignore-accents");
-  }
-  
-}
+} }
 
