@@ -1,6 +1,6 @@
-// Copyright 2000 by Kevin Atkinson under the terms of the LGPL
+// Copyright 2000-2005 by Kevin Atkinson under the terms of the LGPL
 
-// suggest.cc Suggestion code for Aspell
+// suggest.cpp Suggestion code for Aspell
 
 // The magic behind my spell checker comes from merging Lawrence
 // Philips excellent metaphone algorithm and Ispell's near miss
@@ -32,6 +32,9 @@
 // of how the words sounds. It is not the phoneme of the word by any
 // means.  For more information on the metaphone algorithm please see
 // the file metaphone.cc which included a detailed description of it.
+//
+// NOTE: It is assumed that that strlen(soundslike) <= strlen(word)
+//       for any possible word
 
 // POSSIBLE OPTIMIZATION:
 //   store the number of letters that are the same as the previous 
@@ -44,7 +47,7 @@
 #include "speller_impl.hpp"
 #include "asuggest.hpp"
 #include "basic_list.hpp"
-#include "clone_ptr-t.hpp"
+#include "clone_ptr.hpp"
 #include "config.hpp"
 #include "data.hpp"
 #include "editdist.hpp"
@@ -52,7 +55,7 @@
 #include "errors.hpp"
 #include "file_data_util.hpp"
 #include "hash-t.hpp"
-#include "language.hpp"
+#include "lang_impl.hpp"
 #include "leditdist.hpp"
 #include "speller_impl.hpp"
 #include "stack_ptr.hpp"
@@ -60,11 +63,13 @@
 #include "vararray.hpp"
 #include "string_list.hpp"
 
+#include "gettext.h"
+
 //#include "iostream.hpp"
 //#define DEBUG_SUGGEST
 
-using namespace aspeller;
-using namespace acommon;
+using namespace aspell::sp;
+using namespace aspell;
 using namespace std;
 
 namespace {
@@ -97,6 +102,7 @@ namespace {
   struct ScoreWordSound {
     char * word;
     char * word_clean;
+    //unsigned word_size;
     const char * soundslike;
     int           score;
     int           word_score;
@@ -134,12 +140,12 @@ namespace {
  
   class Score {
   protected:
-    const Language * lang;
+    const LangImpl * lang;
     OriginalWord     original;
     const SuggestParms * parms;
 
   public:
-    Score(const Language *l, const String &w, const SuggestParms * p)
+    Score(const LangImpl *l, const String &w, const SuggestParms * p)
       : lang(l), original(), parms(p)
     {
       original.word = w;
@@ -162,7 +168,7 @@ namespace {
     int threshold;
     int try_harder;
 
-    EditDist (* edit_dist_fun)(const char *, const char *, 
+    EditDist (* edit_dist_fun)(const char *, const char *,
                                const EditDistanceWeights &);
 
     unsigned int max_word_length;
@@ -180,7 +186,7 @@ namespace {
     static const bool do_count = true;
     static const bool dont_count = false;
 
-    CheckInfo check_info[8];
+    IntrCheckInfo check_info[8];
 
     void commit_temp(const char * b) {
       if (temp_end) {
@@ -206,10 +212,10 @@ namespace {
       commit_temp(sl);
       return sl;}
 
-    char * Working::form_word(CheckInfo & ci);
+    MutableString form_word(IntrCheckInfo & ci);
     void try_word_n(ParmString str, int score);
-    bool check_word_s(ParmString word, CheckInfo * ci);
-    unsigned check_word(char * word, char * word_end, CheckInfo * ci,
+    bool check_word_s(ParmString word, IntrCheckInfo * ci);
+    unsigned check_word(char * word, char * word_end, IntrCheckInfo * ci,
                         /* it WILL modify word */
                         unsigned pos = 1);
     void try_word_c(char * word, char * word_end, int score);
@@ -275,7 +281,7 @@ namespace {
     void fine_tune_score();
     void transfer();
   public:
-    Working(SpellerImpl * m, const Language *l,
+    Working(SpellerImpl * m, const LangImpl *l,
 	    const String & w, const SuggestParms *  p)
       : Score(l,w,p), threshold(1), max_word_length(0), sp(m) {
       memset(check_info, 0, sizeof(check_info));
@@ -285,6 +291,9 @@ namespace {
 
   void Working::get_suggestions(NearMissesFinal & sug) {
 
+    if (original.word.size() * parms->edit_distance_weights.max >= 0x8000)
+      return; // to prevent overflow in the editdist functions
+
     near_misses_final = & sug;
 
     try_split();
@@ -369,10 +378,11 @@ namespace {
     transfer();
   }
 
-  // forms a word from into the buffer by growing the temporary
-  // string in "buffer".  Returns a pointer to beginning of
-  // the string
-  char * Working::form_word(CheckInfo & ci) 
+  // Forms a word by combining IntrCheckInfo fields.
+  // Will grow the grow the temp in the buffer.  The final
+  // word must be null terminated and commited.
+  // It returns a MutableString of what was appended to the buffer.
+  MutableString Working::form_word(IntrCheckInfo & ci) 
   {
     size_t slen = ci.word.size() - ci.pre_strip_len - ci.suf_strip_len;
     size_t wlen = slen + ci.pre_add_len + ci.suf_add_len;
@@ -382,7 +392,7 @@ namespace {
     memcpy(tmp + ci.pre_add_len, ci.word.str() + ci.pre_strip_len, slen);
     if (ci.suf_add_len) 
       memcpy(tmp + ci.pre_add_len + slen, ci.suf_add, ci.suf_add_len);
-    return tmp;
+    return MutableString(tmp,wlen);
   }
 
   void Working::try_word_n(ParmString str, int score)  
@@ -399,17 +409,19 @@ namespace {
         add_nearmiss(i, sw, 0, score, -1, do_count);
     }
     if (sp->affix_compress) {
-      CheckInfo ci; memset(&ci, 0, sizeof(ci));
+      IntrCheckInfo ci; memset(&ci, 0, sizeof(ci));
       bool res = lang->affix()->affix_check(LookupInfo(sp, LookupInfo::Clean), str, ci, 0);
       if (!res) return;
-      char * tmp = form_word(ci);
+      form_word(ci);
       char * end = (char *)buffer.grow_temp(1);
+      char * tmp = (char *)buffer.temp_ptr();
+      buffer.commit_temp();
       *end = '\0';
       add_nearmiss(tmp, end - tmp, 0, 0, score, -1, do_count);
     }
   }
 
-  bool Working::check_word_s(ParmString word, CheckInfo * ci)
+  bool Working::check_word_s(ParmString word, IntrCheckInfo * ci)
   {
     WordEntry sw;
     for (SpellerImpl::WS::const_iterator i = sp->suggest_ws.begin();
@@ -428,7 +440,7 @@ namespace {
     return false;
   }
 
-  unsigned Working::check_word(char * word, char * word_end,  CheckInfo * ci,
+  unsigned Working::check_word(char * word, char * word_end,  IntrCheckInfo * ci,
                           /* it WILL modify word */
                           unsigned pos)
   {
@@ -447,7 +459,7 @@ namespace {
       res = check_word(i, word_end, ci + 1, pos + 1);
       if (res) return res;
     }
-    memset(ci, 0, sizeof(CheckInfo));
+    memset(ci, 0, sizeof(IntrCheckInfo));
     return 0;
   }
 
@@ -458,19 +470,20 @@ namespace {
     //CERR.printf(">%s\n", word);
     if (!res) return;
     buffer.abort_temp();
-    char * tmp = form_word(check_info[0]);
-    CasePattern cp = lang->case_pattern(tmp);
+    MutableString tmp = form_word(check_info[0]);
+    CasePattern cp = lang->case_pattern(tmp, tmp.size);
     for (unsigned i = 1; i <= res; ++i) {
       char * t = form_word(check_info[i]);
       if (cp == FirstUpper && lang->is_lower(t[1])) 
         t[0] = lang->to_lower(t[0]);
     }
     char * end = (char *)buffer.grow_temp(1);
+    char * beg = (char *)buffer.temp_ptr(); // since the orignal string may of moved
     *end = 0;
     buffer.commit_temp();
-    add_nearmiss(tmp, end - tmp, 0, 0, score, -1, do_count);
+    add_nearmiss(beg, end - beg, 0, 0, score, -1, do_count);
     //CERR.printl(tmp);
-    memset(check_info, 0, sizeof(CheckInfo)*res);
+    memset(check_info, 0, sizeof(IntrCheckInfo)*res);
   }
 
   void Working::add_nearmiss(char * word, unsigned word_size,
@@ -479,6 +492,9 @@ namespace {
                              int w_score, int sl_score, 
                              bool count, WordEntry * rl)
   {
+    if (word_size * parms->edit_distance_weights.max >= 0x8000) 
+      return; // to prevent overflow in the editdist functions
+
     if (w_score < 0) w_score = LARGE_NUM;
     if (sl_score < 0) sl_score = LARGE_NUM;
     if (!sp->have_soundslike) {
@@ -490,6 +506,7 @@ namespace {
     ScoreWordSound & d = near_misses.front();
     d.word = word;
     d.soundslike = sl;
+    //d.word_size = word_size;
     
     if (parms->use_typo_analysis) {
       unsigned int l = word_size;
@@ -527,7 +544,7 @@ namespace {
     add_nearmiss(buffer.dup(ParmString(w.word, w.word_size)), 
                  w.word_size, w.word_info, 
                  sl,
-                 w_score, sl_score, count);
+                 w_score, sl_score, count, repl);
   }
   
   void Working::add_nearmiss(SpellerImpl::WS::const_iterator i,
@@ -557,7 +574,7 @@ namespace {
       new_word[i+1] = new_word[i];
       new_word[i] = '\0';
       
-      if (sp->check(new_word) && sp->check(new_word + i + 1)) {
+      if (sp->check(new_word).data && sp->check(new_word + i + 1).data) {
         for (size_t j = 0; j != parms->split_chars.size(); ++j)
         {
           new_word[i] = parms->split_chars[j];
@@ -598,7 +615,7 @@ namespace {
     
     // Interchange two adjacent letters.
     
-    for (i = 0; i+1 != orig.size(); ++i) {
+    for (i = 0; i+1 < orig.size(); ++i) {
       a = new_word[i];
       b = new_word[i+1];
       new_word[i] = b;
@@ -652,16 +669,16 @@ namespace {
     for (; !w.at_end(); w.adv()) {
       
       add_nearmiss(i, w, sl, -1, score);
-
+      
       if (w.aff[0]) {
         String sl_buf;
         temp_buffer.reset();
         WordAff * exp_list;
-        exp_list = lang->affix()->expand(w.word, w.aff, temp_buffer);
-        for (WordAff * p = exp_list->next; p; p = p->next)
-          add_nearmiss(i, p, 0, -1, -1);
+          exp_list = lang->affix()->expand(w.word, w.aff, temp_buffer);
+          for (WordAff * p = exp_list->next; p; p = p->next)
+            add_nearmiss(i, p, 0, -1, -1);
       }
-
+      
     }
   }
 
@@ -682,12 +699,13 @@ namespace {
          i != sp->suggest_ws.end();
          ++i) 
     {
+      //CERR.printf(">>%p %s\n", *i, typeid(**i).name());
       StackPtr<SoundslikeEnumeration> els((*i)->soundslike_elements());
 
       while ( (sw = els->next(stopped_at)) ) {
 
         //CERR.printf("[%s (%d) %d]\n", sw->word, sw->word_size, sw->what);
-        assert(strlen(sw->word) == sw->word_size);
+        //assert(strlen(sw->word) == sw->word_size);
           
         if (sw->what != WordEntry::Word) {
           sl = sw->word;
@@ -781,7 +799,7 @@ namespace {
 #ifdef DEBUG_SUGGEST
     COUT.printf("will try soundslike: %s\n", sls.back());
 #endif
-    for (const aspeller::CheckInfo * ci = gi.head;
+    for (const IntrCheckInfo * ci = gi.head;
          ci; 
          ci = ci->next) 
     {
@@ -908,12 +926,12 @@ namespace {
     int min_score = 0;
     int count = 0;
 
-    for (NearMisses::iterator i = scored_near_misses.begin();
-         i != scored_near_misses.end(); ++i)
+    for (NearMisses::iterator mi = scored_near_misses.begin();
+         mi != scored_near_misses.end(); ++mi)
     {
-      if (!i->soundslike)
-        i->soundslike = to_soundslike(i->word, strlen(i->word));
-      already_have.insert(i->soundslike);
+      if (!mi->soundslike)
+        mi->soundslike = to_soundslike(mi->word, strlen(mi->word));
+      already_have.insert(mi->soundslike);
     }
 
     for (SpellerImpl::WS::const_iterator i = sp->suggest_ws.begin();
@@ -961,12 +979,12 @@ namespace {
       }
     }
     
-    for (Candidates::iterator i = candidates.begin();
-         i != candidates.end();
-         ++i)
+    for (Candidates::iterator ci = candidates.begin();
+         ci != candidates.end();
+         ++ci)
     {
-      //COUT.printf("ngram: %s %d\n", i->soundslike, i->score);
-      add_sound(i->i, &i->info, i->soundslike);
+      //COUT.printf("ngram: %s %d\n", candidate->soundslike, candidate->score);
+      add_sound(ci->i, &ci->info, ci->soundslike);
     }
   }
   
@@ -992,7 +1010,7 @@ namespace {
     int try_for = (parms->word_weight*parms->edit_distance_weights.max)/100;
     while (true) {
       try_for += (parms->word_weight*parms->edit_distance_weights.max)/100;
-	
+
       // put all pairs whose score <= initial_limit*max_weight
       // into the scored list
 
@@ -1005,17 +1023,16 @@ namespace {
         //            i->word_score, i->soundslike_score);
 
         if (i->word_score >= LARGE_NUM) {
-
-            int sl_score = i->soundslike_score < LARGE_NUM ? i->soundslike_score : 0;
-            int level = needed_level(try_for, sl_score);
-            
-            if (level >= int(sl_score/parms->edit_distance_weights.min))
-              i->word_score = edit_distance(original.clean,
-                                            i->word_clean,
-                                            level, level,
-                                            parms->edit_distance_weights);
+          int sl_score = i->soundslike_score < LARGE_NUM ? i->soundslike_score : 0;
+          int level = needed_level(try_for, sl_score);
+          
+          if (level >= int(sl_score/parms->edit_distance_weights.min)) 
+            i->word_score = edit_distance(original.clean,
+                                          i->word_clean,
+                                          level, level,
+                                          parms->edit_distance_weights);
         }
-
+        
         if (i->word_score >= LARGE_NUM) goto cont1;
 
         if (i->soundslike_score >= LARGE_NUM) 
@@ -1139,6 +1156,7 @@ namespace {
 #  ifdef DEBUG_SUGGEST
     COUT << "Size of scored: " << scored_near_misses.size() << "\n";
     COUT << "Size of ! scored: " << near_misses.size() << "\n";
+    COUT << "Try Harder: " << try_harder << "\n";
 #  endif
   }
 
@@ -1204,13 +1222,12 @@ namespace {
       if (i->repl_list != 0) {
  	String::size_type pos;
 	do {
-          abort();
  	  dup_pair = duplicates_check.insert(fix_case(i->repl_list->word, buf));
  	  if (dup_pair.second && 
  	      ((pos = dup_pair.first->find(' '), pos == String::npos)
- 	       ? (bool)sp->check(*dup_pair.first)
- 	       : (sp->check((String)dup_pair.first->substr(0,pos)) 
- 		  && sp->check((String)dup_pair.first->substr(pos+1))) ))
+ 	       ? sp->check(*dup_pair.first).data
+ 	       : (sp->check((String)dup_pair.first->substr(0,pos)).data 
+ 		  && sp->check((String)dup_pair.first->substr(pos+1)).data) ))
  	    near_misses_final->push_back(*dup_pair.first);
  	} while (i->repl_list->adv());
       } else {
@@ -1294,7 +1311,7 @@ namespace {
     if (keyboard == "none")
       parms_.use_typo_analysis = false;
     else
-      RET_ON_ERR(aspeller::setup(parms_.ti, m->config(), &m->lang(), keyboard));
+      RET_ON_ERR(aspell::sp::setup(parms_.ti, m->config(), &m->lang(), keyboard));
 
     return no_err;
   }
@@ -1315,7 +1332,7 @@ namespace {
   
 }
 
-namespace aspeller {
+namespace aspell { namespace sp {
   PosibErr<Suggest *> new_default_suggest(SpellerImpl * m) {
     StackPtr<SuggestImpl> s(new SuggestImpl);
     RET_ON_ERR(s->setup(m));
@@ -1354,14 +1371,18 @@ namespace aspeller {
     try_one_edit_word = sp->soundslike_root_only || sp->unconditional_run_together_;
     check_after_one_edit_word = false;
     ngram_threshold = 2;
-    if (mode == "ultra" || mode == "fast") {
+    if (mode == "ultra") {
+      try_scan_1 = true;
+      try_scan_2 = false;
+      try_ngram = false;
+    } else if (mode == "fast") {
       try_scan_1 = true;
       try_scan_2 = false;
       try_ngram = false;
     } else if (mode == "normal") {
       try_scan_1 = true;
       try_scan_2 = true;
-      try_ngram = true;
+      try_ngram = false;
     } else if (mode == "slow") {
       try_scan_1 = false;
       try_scan_2 = true;
@@ -1378,14 +1399,14 @@ namespace aspeller {
       limit = 1000;
       ngram_threshold = 1;
     } else {
-      return make_err(bad_value, "sug-mode", mode, "one of ultra, fast, normal, slow, or bad-spellers");
+      return make_err(bad_value, "sug-mode", mode, _("one of ultra, fast, normal, slow, or bad-spellers"));
     }
     if (!sp->have_soundslike) {
       if (try_scan_2) {
         try_scan_1 = false;
       } else if (try_scan_1) {
         try_one_edit_word = true;
-        //check_after_one_edit_word = true;
+        check_after_one_edit_word = true;
         try_scan_1 = false;
       }
     }
@@ -1406,4 +1427,4 @@ namespace aspeller {
     }
     word_weight = 100 - soundslike_weight;
   }
-}
+} }
