@@ -1,5 +1,5 @@
 // This file is part of The New Aspell
-// Copyright (C) 2002 by Kevin Atkinson under the GNU LGPL license
+// Copyright (C) 2002,2003,2004 by Kevin Atkinson under the GNU LGPL license
 // version 2.0 or 2.1.  You should have received a copy of the LGPL
 // license along with this library if you did not you can find
 // it at http://www.gnu.org/.
@@ -19,6 +19,9 @@
 
 #ifdef USE_LOCALE
 # include <locale.h>
+#endif
+
+#ifdef HAVE_LANGINFO_CODESET
 # include <langinfo.h>
 #endif
 
@@ -31,15 +34,15 @@
 # include <fcntl.h>
 #endif
 
+#include "file_util.hpp"
+#include "fstream.hpp"
 #include "asc_ctype.hpp"
 #include "check_funs.hpp"
 #include "config.hpp"
 #include "convert.hpp"
-#include "document_checker.hpp"
+#include "checker.hpp"
 #include "enumeration.hpp"
 #include "errors.hpp"
-#include "file_util.hpp"
-#include "fstream.hpp"
 #include "info.hpp"
 #include "iostream.hpp"
 #include "posib_err.hpp"
@@ -53,9 +56,12 @@
 #include "speller_impl.hpp"
 #include "data.hpp"
 
-using namespace acommon;
+#include "hash-t.hpp"
+#include "hash_fun.hpp"
 
-using aspeller::Conv;
+#include "gettext.h"
+
+using namespace aspell;
 
 // action functions declarations
 
@@ -81,6 +87,7 @@ void soundslike();
 void munch();
 void expand();
 void combine();
+void munch_list();
 void dump_affix();
 
 void print_error(ParmString msg)
@@ -128,7 +135,7 @@ typedef Config         Options;
 enum Action {do_create, do_merge, do_dump, do_test, do_other};
 
 Args              args;
-StackPtr<Options> options(new_config());
+StackPtr<Options> options;
 Action            action  = do_other;
 
 struct PossibleOption {
@@ -147,6 +154,7 @@ const PossibleOption possible_options[] = {
   OPTION("personal",         'p', 1),
   OPTION("ignore",           'W', 1),
   OPTION("lang",             'l', 1),
+  OPTION("encoding",         'T', 1),
   OPTION("backup",           'b', 0),
   OPTION("dont-backup",      'x', 0),
   OPTION("run-together",     'C', 0),
@@ -154,12 +162,14 @@ const PossibleOption possible_options[] = {
   OPTION("guess",            'm', 0),
   OPTION("dont-guess",       'P', 0),
   
-  COMMAND("version",   'v', 0),
+  COMMAND("usage",     '?',  0),
   COMMAND("help",      '\0', 0),
-  COMMAND("usage",     '?', 0),
+  COMMAND("version",   'v',  0),
   COMMAND("config",    '\0', 0),
-  COMMAND("check",     'c', 0),
-  COMMAND("pipe",      'a', 0),
+  COMMAND("dicts",     '\0', 0),
+  COMMAND("check",     'c',  0),
+  COMMAND("pipe",      'a',  0),
+  COMMAND("list",      '\0', 0),
   COMMAND("conv",      '\0', 2),
   COMMAND("norm",      '\0', 1),
   COMMAND("filter",    '\0', 0),
@@ -167,18 +177,16 @@ const PossibleOption possible_options[] = {
   COMMAND("munch",     '\0', 0),
   COMMAND("expand",    '\0', 0),
   COMMAND("combine",   '\0', 0),
-  COMMAND("list",      '\0', 0),
-  COMMAND("dicts",     '\0', 0),
+  COMMAND("munch-list",'\0', 0),
+  COMMAND("clean",     '\0', 0),
   COMMAND("filters",   '\0', 0),
   COMMAND("modes",     '\0', 0),
-  COMMAND("clean",     '\0', 0),
 
   COMMAND("dump",   '\0', 1),
   COMMAND("create", '\0', 1),
   COMMAND("merge",  '\0', 1),
 
-  ISPELL_COMP('n',0), ISPELL_COMP('S',0), 
-  ISPELL_COMP('w',1), ISPELL_COMP('T',1),
+  ISPELL_COMP('S',0), ISPELL_COMP('w',1),
 
   {"",'\0'}, {"",'\0'}
 };
@@ -194,9 +202,10 @@ static const ModeAbrv mode_abrvs[] = {
   {'e', "mode=email", N_("enter Email mode.")},
   {'H', "mode=html",  N_("enter HTML mode.")},
   {'t', "mode=tex",   N_("enter TeX mode.")},
+  {'n', "mode=nroff", N_("enter Nroff mode.")}
 };
 
-static const ModeAbrv *  mode_abrvs_end = mode_abrvs + 3;
+static const ModeAbrv *  mode_abrvs_end = mode_abrvs + 4;
 
 const PossibleOption * find_option(char c) {
   const PossibleOption * i = possible_options;
@@ -234,6 +243,8 @@ Conv uiconv;
 
 int main (int argc, const char *argv[]) 
 {
+  options = new_config(); // this needs to be here becuase of a bug
+                          // with static initlizers on Darwin.
 #ifdef USE_LOCALE
   setlocale (LC_ALL, "");
 #endif
@@ -309,7 +320,7 @@ int main (int argc, const char *argv[])
           num_parms = 0;
           i += 1;
         } else {
-          print_error(_("You must specify a parameter for %s"), argv[i]);
+          print_error(_("You must specify a parameter for \"%s\"."), argv[i]);
           return 1;
         }
       } else {
@@ -343,7 +354,7 @@ int main (int argc, const char *argv[])
   const char * codeset = 0;
 #ifdef HAVE_LANGINFO_CODESET
   codeset = nl_langinfo(CODESET);
-  if (is_ascii_enc(codeset)) codeset = 0;
+  if (ascii_encoding(*options, codeset)) codeset = 0;
 #endif
 
 // #ifdef USE_LOCALE
@@ -362,11 +373,24 @@ int main (int argc, const char *argv[])
     return 1;
   }
 
+  String action_str = args.front();
+  args.pop_front();
+  const PossibleOption * action_opt = find_option(action_str.str());
+  if (!action_opt->is_command) {
+    print_error(_("Unknown Action: %s"),  action_str);
+    return 1;
+  } else if (action_opt->num_arg == 1 && args.empty()) {
+    print_error(_("You must specify a parameter for \"%s\"."), action_str);
+    return 1;
+  } else if (action_opt->num_arg > (int)args.size()) {
+    CERR.printf(_("Error: You must specify at least %d parameters for \"%s\".\n"), 
+                action_opt->num_arg, action_str.str());
+    return 1;
+  }
+
   //
   // perform the requested action
   //
-  String action_str = args.front();
-  args.pop_front();
   if (action_str == "usage")
     print_help();
   else if (action_str == "help")
@@ -397,18 +421,22 @@ int main (int argc, const char *argv[])
     expand();
   else if (action_str == "combine")
     combine();
+  else if (action_str == "munch-list")
+    munch_list();
+  else if (action_str == "clean")
+    clean();
+  else if (action_str == "filters")
+    filters();
+  else if (action_str == "modes")
+    modes();
   else if (action_str == "dump")
     action = do_dump;
   else if (action_str == "create")
     action = do_create;
   else if (action_str == "merge")
     action = do_merge;
-  else if (action_str == "clean")
-    clean();
-  else {
-    print_error(_("Unknown Action: %s"),  action_str);
-    return 1;
-  }
+  else
+    abort(); // this should not happen
 
   if (action != do_other) {
     if (args.empty()) {
@@ -452,14 +480,15 @@ int main (int argc, const char *argv[])
 //
 
   
-static Convert * setup_conv(const aspeller::Language * lang,
-                                      Config * config)
+static SimpleConvert * setup_conv(const sp::LangImpl * lang,
+                                  Config * config)
 {
   if (config->retrieve("encoding") != "none") {
-    PosibErr<Convert *> pe = new_convert_if_needed(*config,
-                                                   lang->charmap(),
-                                                   config->retrieve("encoding"),
-                                                   NormTo);
+    PosibErr<SimpleConvert *> pe 
+      = new_simple_convert_if_needed(*config,
+                                     lang->charmap(),
+                                     config->retrieve("encoding"),
+                                     NormTo);
     if (pe.has_err()) {print_error(pe.get_err()->mesg); exit(1);}
     return pe.data;
   } else {
@@ -467,14 +496,15 @@ static Convert * setup_conv(const aspeller::Language * lang,
   }
 }
  
-static Convert * setup_conv(Config * config,
-                            const aspeller::Language * lang)
+static SimpleConvert * setup_conv(Config * config,
+                                  const sp::LangImpl * lang)
 {
   if (config->retrieve("encoding") != "none") {
-    PosibErr<Convert *> pe = new_convert_if_needed(*config,
-                                                   config->retrieve("encoding"),
-                                                   lang->charmap(),
-                                                   NormFrom);
+    PosibErr<SimpleConvert *> pe 
+      = new_simple_convert_if_needed(*config,
+                                     config->retrieve("encoding"),
+                                     lang->charmap(),
+                                     NormFrom);
     if (pe.has_err()) {print_error(pe.get_err()->mesg); exit(1);}
     return pe.data;
   } else {
@@ -482,19 +512,19 @@ static Convert * setup_conv(Config * config,
   }
 }
 
-void setup_display_conv()
+void setup_display_conv(Config * config)
 {
   const char * gettext_enc = 0;
   const char * env_enc = 0;
-  String doc_enc = options->retrieve("encoding");
+  String doc_enc = config->retrieve("encoding");
   String enc;
 #ifdef ENABLE_NLS
   gettext_enc = bind_textdomain_codeset("aspell", 0);
-  if (gettext_enc && is_ascii_enc(gettext_enc)) gettext_enc = 0;
+  if (ascii_encoding(*options,gettext_enc)) gettext_enc = 0;
 #endif
 #ifdef HAVE_LANGINFO_CODESET
   env_enc = nl_langinfo(CODESET);
-  if (is_ascii_enc(env_enc)) env_enc = 0;
+  if (ascii_encoding(*options, env_enc)) env_enc = 0;
 #endif
   if (gettext_enc && env_enc && strcmp(gettext_enc,env_enc) != 0) 
   {
@@ -508,11 +538,43 @@ void setup_display_conv()
   else
     enc = doc_enc;
 
-  fprintf(stderr, "'%s' '%s' '%s' '%s'\n", gettext_enc, env_enc, doc_enc.str(), enc.str());
   EXIT_ON_ERR(dconv.setup(*options, doc_enc, enc, NormNone));
   EXIT_ON_ERR(uiconv.setup(*options, enc, doc_enc, NormNone));
 }
 
+AspellSpeller * new_speller()
+{
+  AspellCanHaveError * ret 
+    = new_aspell_speller(reinterpret_cast<AspellConfig *>(options.get()));
+  if (aspell_error(ret)) {
+    print_error(aspell_error_message(ret));
+    exit(1);
+  }
+  return to_aspell_speller(ret);
+}
+
+AspellChecker * new_checker(AspellSpeller * sp)
+{
+  AspellCanHaveError * ret = new_aspell_checker(sp);
+  if (aspell_error(ret)) {
+    print_error(aspell_error_message(ret));
+    exit(1);
+  }
+  return to_aspell_checker(ret);
+}
+
+AspellLanguage * new_language()
+{
+  AspellCanHaveError * ret 
+    = new_aspell_language(reinterpret_cast<AspellConfig *>(options.get()));
+  if (aspell_error(ret)) {
+    print_error(aspell_error_message(ret));
+    exit(1);
+  }
+  return to_aspell_language(ret);
+
+}
+
 
 ///////////////////////////
 //
@@ -622,35 +684,6 @@ void print_elements(const AspellWordList * wl) {
   COUT.printf("%u: %s\n", count, line.c_str());
 }
 
-struct StatusFunInf 
-{
-  aspeller::SpellerImpl * real_speller;
-  bool verbose;
-};
-
-void status_fun(void * d, Token, int correct)
-{
-  StatusFunInf * p = static_cast<StatusFunInf *>(d);
-  if (p->verbose && correct) {
-    const CheckInfo * ci = p->real_speller->check_info();
-    if (ci->compound)
-      COUT.put("-\n");
-    else if (ci->pre_flag || ci->suf_flag)
-      COUT.printf("+ %s\n", ci->word.str());
-    else
-      COUT.put("*\n");
-  }
-}
-
-DocumentChecker * new_checker(AspellSpeller * speller, 
-			      StatusFunInf & status_fun_inf) 
-{
-  EXIT_ON_ERR_SET(new_document_checker(reinterpret_cast<Speller *>(speller)),
-		  StackPtr<DocumentChecker>, checker);
-  checker->set_status_fun(status_fun, &status_fun_inf);
-  return checker.release();
-}
-
 #define BREAK_ON_SPELLER_ERR\
   do {if (aspell_speller_error(speller)) {\
     print_error(aspell_speller_error_message(speller)); break;\
@@ -658,41 +691,37 @@ DocumentChecker * new_checker(AspellSpeller * speller,
 
 void pipe() 
 {
-#ifndef WIN32
+#if ! defined(WIN32) && ! defined(_WIN32)
   // set up stdin and stdout to be line buffered
   assert(setvbuf(stdin, 0, _IOLBF, 0) == 0); 
   assert(setvbuf(stdout, 0, _IOLBF, 0) == 0);
 #endif
 
-  bool terse_mode = true;
+  bool terse_mode = false;
   bool do_time = options->retrieve_bool("time");
   bool suggest = options->retrieve_bool("suggest");
   bool include_guesses = options->retrieve_bool("guess");
   clock_t start,finish;
+
+  if (!options->have("mode") && !options->have("filter")) {
+    PosibErrBase err(options->replace("mode", "nroff"));
+    if (err.has_err()) 
+      CERR.printf(_("WARNING: Unable to enter Nroff mode: %s\n"),
+                  err.get_err()->mesg);
+  }
+
   start = clock();
   
-  AspellCanHaveError * ret 
-    = new_aspell_speller(reinterpret_cast<AspellConfig *>(options.get()));
-  if (aspell_error(ret)) {
-    print_error(aspell_error_message(ret));
-    exit(1);
-  }
-  AspellSpeller * speller = to_aspell_speller(ret);
-  aspeller::SpellerImpl * real_speller = reinterpret_cast<aspeller::SpellerImpl *>(speller);
+  AspellSpeller * speller = new_speller();
+  aspell::sp::SpellerImpl * real_speller = reinterpret_cast<aspell::sp::SpellerImpl *>(speller);
   Config * config = real_speller->config();
-  Conv iconv(setup_conv(config, &real_speller->lang()));
-  Conv oconv(setup_conv(&real_speller->lang(), config));
   MBLen mb_len;
-  if (!config->retrieve_bool("byte-offsets")) 
+  if (!config->retrieve_bool("byte-offsets").data) 
     mb_len.setup(*config, config->retrieve("encoding"));
   if (do_time)
     COUT << _("Time to load word list: ")
          << (clock() - start)/(double)CLOCKS_PER_SEC << "\n";
-  StatusFunInf status_fun_inf;
-  status_fun_inf.real_speller = real_speller;
-  bool & print_star = status_fun_inf.verbose;
-  print_star = true;
-  StackPtr<DocumentChecker> checker(new_checker(speller, status_fun_inf));
+  AspellChecker * checker = new_checker(speller);
   int c;
   const char * w;
   CharVector buf;
@@ -725,9 +754,7 @@ void pipe()
       break;
     case '&':
       word = trim_wspace(line + 1);
-      aspell_speller_add_to_personal
-	(speller, 
-	 real_speller->to_lower(word), -1);
+      aspell_speller_add_lower_to_personal(speller, word, -1);
       BREAK_ON_SPELLER_ERR;
       break;
     case '@':
@@ -744,25 +771,19 @@ void pipe()
       err = config->replace("mode", word);
       if (err.get_err())
 	config->replace("mode", "tex");
-      reload_filters(real_speller);
-      checker.del();
-      checker = new_checker(speller, status_fun_inf);
-      break;
+      goto reload_filters;
     case '-':
       config->remove("filter");
-      reload_filters(real_speller);
-      checker.del();
-      checker = new_checker(speller, status_fun_inf);
-      break;
+      goto reload_filters;
     case '~':
-      break;
+      word = trim_wspace(line + 1);
+      BREAK_ON_ERR(config->replace("encoding", word));
+      goto reload_filters;
     case '!':
       terse_mode = true;
-      print_star = false;
       break;
     case '%':
       terse_mode = false;
-      print_star = true;
       break;
     case '$':
       if (line[1] == '$') {
@@ -817,80 +838,84 @@ void pipe()
     default:
       line0 = line;
       line += ignore;
-      checker->process(line, strlen(line));
-      while (Token token = checker->next_misspelling()) {
-	word = line + token.offset;
-	word[token.len] = '\0';
-        const char * cword = iconv(word);
-        String guesses, guess;
-        const CheckInfo * ci = real_speller->check_info();
-        aspeller::CasePattern casep 
-          = real_speller->lang().case_pattern(cword);
-        while (ci) {
-          guess.clear();
-          if (ci->pre_add && ci->pre_add[0])
-            guess.append(ci->pre_add, ci->pre_add_len).append('+');
-          guess.append(ci->word);
-          if (ci->pre_strip_len > 0) 
-            guess.append('-').append(ci->word.str(), ci->pre_strip_len);
-          if (ci->suf_strip_len > 0) 
-            guess.append('-').append(ci->word.str() - ci->suf_strip_len, ci->suf_strip_len);
-          if (ci->suf_add && ci->suf_add[0])
-            guess.append('+').append(ci->suf_add, ci->suf_add_len);
-          real_speller->lang().fix_case(casep, guess.data(), guess.data());
-          guesses << ", " << oconv(guess.str());
-          ci = ci->next;
+      aspell_checker_process(checker, line, strlen(line), 0, 0);
+      const AspellCheckerToken * token = 0;
+      while (token = aspell_checker_next(checker), token) {
+        if (token->correct) {
+          if (!terse_mode) {
+            const AspellCheckInfo * ci = aspell_speller_check_info(speller);
+            COUT.put(ci->str);
+            COUT.put('\n');
+          }
+        } else {
+          word = line + token->begin.offset;
+          unsigned word_len = token->end.offset - token->begin.offset;
+          word[word_len] = '\0';
+          String guesses;
+          const AspellCheckInfo * ci = aspell_speller_check_info(speller);
+          while (ci) {
+            guesses << ", " << ci->str;
+            ci = ci->next;
+          }
+          start = clock();
+          const AspellWordList * suggestions = 0;
+          if (suggest) 
+            suggestions = aspell_speller_suggest(speller, word, -1);
+          finish = clock();
+          unsigned offset = mb_len(line0, token->begin.offset + ignore);
+          if (suggestions && !aspell_word_list_empty(suggestions)) 
+          {
+            COUT.printf("& %s %u %u:", word, 
+                        aspell_word_list_size(suggestions), offset);
+            AspellStringEnumeration * els 
+              = aspell_word_list_elements(suggestions);
+            if (options->retrieve_bool("reverse")) {
+              Vector<String> sugs;
+              sugs.reserve(aspell_word_list_size(suggestions));
+              while ( ( w = aspell_string_enumeration_next(els)) != 0)
+                sugs.push_back(w);
+              Vector<String>::reverse_iterator i = sugs.rbegin();
+              while (true) {
+                COUT.printf(" %s", i->c_str());
+                ++i;
+                if (i == sugs.rend()) break;
+                COUT.put(',');
+              }
+            } else {
+              while ( ( w = aspell_string_enumeration_next(els)) != 0) {
+                COUT.printf(" %s%s", w, 
+                            aspell_string_enumeration_at_end(els) ? "" : ",");
+              }
+            }
+            delete_aspell_string_enumeration(els);
+            if (include_guesses)
+              COUT.put(guesses);
+            COUT.put('\n');
+          } else {
+            if (guesses.empty())
+              COUT.printf("# %s %u\n", word, offset);
+            else
+              COUT.printf("? %s 0 %u: %s\n", word, offset,
+                          guesses.c_str() + 2);
+          }
+          if (do_time)
+            COUT.printf(_("Suggestion Time: %f\n"), 
+                        (finish-start)/(double)CLOCKS_PER_SEC);
         }
-	start = clock();
-        const AspellWordList * suggestions = 0;
-        if (suggest) 
-          suggestions = aspell_speller_suggest(speller, word, -1);
-	finish = clock();
-        unsigned offset = mb_len(line0, token.offset + ignore);
-	if (suggestions && !aspell_word_list_empty(suggestions)) 
-        {
-          COUT.printf("& %s %u %u:", word, 
-                      aspell_word_list_size(suggestions), offset);
-	  AspellStringEnumeration * els 
-	    = aspell_word_list_elements(suggestions);
-	  if (options->retrieve_bool("reverse")) {
-	    Vector<String> sugs;
-	    sugs.reserve(aspell_word_list_size(suggestions));
-	    while ( ( w = aspell_string_enumeration_next(els)) != 0)
-	      sugs.push_back(w);
-	    Vector<String>::reverse_iterator i = sugs.rbegin();
-	    while (true) {
-              COUT.printf(" %s", i->c_str());
-	      ++i;
-	      if (i == sugs.rend()) break;
-              COUT.put(',');
-	    }
-	  } else {
-	    while ( ( w = aspell_string_enumeration_next(els)) != 0) {
-              COUT.printf(" %s%s", w, 
-                          aspell_string_enumeration_at_end(els) ? "" : ",");
-	    }
-	  }
-	  delete_aspell_string_enumeration(els);
-          if (include_guesses)
-            COUT.put(guesses);
-	  COUT.put('\n');
-	} else {
-          if (guesses.empty())
-            COUT.printf("# %s %u\n", word, offset);
-          else
-            COUT.printf("? %s 0 %u: %s", word, offset,
-                        guesses.c_str() + 2);
-	}
-	if (do_time)
-          COUT.printf(_("Suggestion Time: %f\n"), 
-                      (finish-start)/(double)CLOCKS_PER_SEC);
       }
       COUT.put('\n');
     }
     if (c == EOF) break;
+    continue;
+  reload_filters:
+    BREAK_ON_ERR(config->replace("encoding", word));
+    BREAK_ON_ERR(reload_filters(real_speller));
+    //checker.del(); // FIXME.  Why?
+    //checker = new_checker(speller, status_fun_inf);
+    continue;
   }
 
+  delete_aspell_checker(checker);
   delete_aspell_speller(speller);
 }
 
@@ -914,7 +939,7 @@ struct Mapping {
 
 void abort_check();
 
-void setup_display_conv();
+void setup_display_conv(Config *);
 
 void check()
 {
@@ -928,8 +953,11 @@ void check()
   if (args.size() == 0) {
     print_error(_("You must specify a file name."));
     exit(-1);
+  } else if (args.size() > 1) {
+    print_error(_("Only one file name may be specified."));
+    exit(-1);
   }
-    
+  
   file_name = args[0];
   new_name = file_name;
   new_name += ".new";
@@ -968,18 +996,10 @@ void check()
     exit(-1);
   }
 
-  AspellCanHaveError * ret 
-    = new_aspell_speller(reinterpret_cast<AspellConfig *>(options.get()));
-  if (aspell_error(ret)) {
-    print_error(aspell_error_message(ret));
-    exit(1);
-  }
+  AspellSpeller * speller = new_speller();
+  AspellChecker * checker = new_checker(speller);
 
-  setup_display_conv();
-
-  AspellSpeller * speller = to_aspell_speller(ret);
-
-  state = new CheckerString(speller,in,out,64);
+  state = new CheckerString(checker, speller,in,out,64);
  
   word_choices = new Choices;
 
@@ -1066,12 +1086,16 @@ void check()
     switch (mapping[choice]) {
     case Exit:
       goto exit_loop;
-    case Abort:
+    case Abort: {
       prompt(_("Are you sure you want to abort (y/n)? "));
       get_choice(choice);
-      if (choice == 'y' || choice == 'Y')
+      /* TRANSLATORS: The user may input any of these characters to say "yes".
+         MUST ONLY CONSIST OF ASCII CHARACTERS. */
+      const char * yes_characters = _("Yy");
+      if (strchr(yes_characters, choice) != 0)
         goto abort_loop;
       goto choice_prompt;
+    }
     case Ignore:
       break;
     case IgnoreAll:
@@ -1081,12 +1105,11 @@ void check()
       aspell_speller_add_to_personal(speller, word, -1);
       break;
     case AddLower:
-      aspell_speller_add_to_personal
-        (speller, 
-         reinterpret_cast<Speller *>(speller)->to_lower(word), -1);
+      aspell_speller_add_lower_to_personal(speller, word, -1);
       break;
     case Replace:
     case ReplaceAll:
+      // the string new_word is in the encoding of the document
       prompt(_("With: "));
       get_line(new_word);
       if (new_word.size() == 0)
@@ -1095,10 +1118,11 @@ void check()
         new_word = sug_con[new_word[0]-'1'];
       state->replace(new_word);
       changed = true;
-      if (mapping[choice] == ReplaceAll)
+      if (mapping[choice] == ReplaceAll && (strcmp(word,new_word.str()) != 0))
         replace_list->replace(word, new_word);
       break;
     default:
+      // the replasments are in the encoding of the document
       if (choice >= '1' && choice < (char)suggestions_size + '1') { 
         state->replace(sug_con[choice-'1']);
         changed = true;
@@ -1112,6 +1136,7 @@ exit_loop:
   {
     aspell_speller_save_all_word_lists(speller);
     state.del(); // to close the file handles
+    delete_aspell_checker(checker);
     delete_aspell_speller(speller);
 
     if (changed) {
@@ -1137,6 +1162,7 @@ exit_loop:
 abort_loop:
   {
     state.del(); // to close the file handles
+    delete_aspell_checker(checker);
     delete_aspell_speller(speller);
 
     remove_file(new_name);
@@ -1227,15 +1253,10 @@ void Mapping::to_ispell()
 
 void list()
 {
-  AspellCanHaveError * ret 
-    = new_aspell_speller(reinterpret_cast<AspellConfig *>(options.get()));
-  if (aspell_error(ret)) {
-    print_error(aspell_error_message(ret));
-    exit(1);
-  }
-  AspellSpeller * speller = to_aspell_speller(ret);
+  AspellSpeller * speller = new_speller();
+  AspellChecker * checker = new_checker(speller);
 
-  state = new CheckerString(speller,stdin,0,64);
+  state = new CheckerString(checker, speller,stdin,0,64);
 
   String word;
  
@@ -1350,7 +1371,7 @@ class IstreamEnumeration : public StringEnumeration {
   String data;
 public:
   IstreamEnumeration(FStream & i) : in(&i) {}
-  IstreamEnumeration * clone() const {
+  StringEnumeration * clone() const {
     return new IstreamEnumeration(*this);
   }
   void assign (const StringEnumeration * other) {
@@ -1370,15 +1391,15 @@ public:
 
 void clean()
 {
-  using namespace aspeller;
+  using namespace aspell::sp;
 
   bool strict = args.size() != 0 && args[0] == "strict";
   
   Config * config = options;
 
-  CachePtr<Language> lang;
+  CachePtr<LangImpl> lang;
   find_language(*config);
-  PosibErr<Language *> res = new_language(*config);
+  PosibErr<LangImpl *> res = new_lang_impl(*config);
   if (res.has_err()) {print_error(res.get_err()->mesg); exit(1);}
   lang.reset(res.data);
   IstreamEnumeration in(CIN);
@@ -1411,9 +1432,9 @@ void clean()
 // master
 //
 
-void dump (aspeller::Dict * lws, Convert * conv) 
+void dump (aspell::sp::Dict * lws, Convert * conv) 
 {
-  using namespace aspeller;
+  using namespace aspell::sp;
 
   switch (lws->basic_type) {
   case Dict::basic_dict:
@@ -1440,8 +1461,20 @@ void dump (aspeller::Dict * lws, Convert * conv)
   }
 }
 
+void dump (AspellSpeller * speller, 
+           const AspellWordList * (* get_word_list)(AspellSpeller *))
+{
+  const AspellWordList * wl = get_word_list(speller);
+  if (!wl) {print_error(aspell_speller_error_message(speller)); return;}
+  AspellStringEnumeration * els = aspell_word_list_elements(wl);
+  const char * w;
+  while ( (w = aspell_string_enumeration_next(els)) != 0 ) {
+    COUT.printl(w);
+  }
+}
+
 void master () {
-  using namespace aspeller;
+  using namespace aspell::sp;
 
   if (args.size() != 0) {
     options->replace("master", args[0].c_str());
@@ -1463,9 +1496,9 @@ void master () {
     
   } else if (action == do_dump) {
 
-    EXIT_ON_ERR_SET(add_data_set(config->retrieve("master-path"), *config), Dict *, d);
-    StackPtr<Convert> conv(setup_conv(d->lang(), config));
-    dump(d, conv);
+    AspellSpeller * speller = new_speller();
+    dump(speller, aspell_speller_main_word_list);
+    delete_aspell_speller(speller);
   }
 }
 
@@ -1475,7 +1508,7 @@ void master () {
 //
 
 void personal () {
-  using namespace aspeller;
+  using namespace aspell::sp;
 
   if (args.size() != 0) {
     EXIT_ON_ERR(options->replace("personal", args[0]));
@@ -1507,20 +1540,10 @@ void personal () {
 
   } else { // action == do_dump
 
-    // FIXME: This is currently broken
+    AspellSpeller * speller = new_speller();
+    dump(speller, aspell_speller_personal_word_list);
+    delete_aspell_speller(speller);
 
-    Config * config = options;
-    Dictionary * per = new_default_writable_dict();
-    per->load(config->retrieve("personal-path"), *config);
-    StackPtr<WordEntryEnumeration> els(per->detailed_elements());
-    StackPtr<Convert> conv(setup_conv(per->lang(), config));
-
-    WordEntry * wi;
-    while (wi = els->next(), wi) {
-      wi->write(COUT,*(per->lang()), conv);
-      COUT.put('\n');
-    }
-    delete per;
   }
 }
 
@@ -1530,7 +1553,7 @@ void personal () {
 //
 
 void repl() {
-  using namespace aspeller;
+  using namespace aspell::sp;
 
   if (args.size() != 0) {
     options->replace("repl", args[0].c_str());
@@ -1595,21 +1618,13 @@ void repl() {
 //
 
 void soundslike() {
-  using namespace aspeller;
-  CachePtr<Language> lang;
-  find_language(*options);
-  PosibErr<Language *> res = new_language(*options);
-  if (res.has_err()) {print_error(res.get_err()->mesg); exit(1);}
-  lang.reset(res.data);
-  Conv iconv(setup_conv(options, lang));
-  Conv oconv(setup_conv(lang, options));
+  AspellLanguage * lang = new_language();
   String word;
-  String sl;
   while (CIN.getline(word)) {
-    const char * w = iconv(word);
-    lang->LangImpl::to_soundslike(sl, w);
-    printf("%s\t%s\n", word.str(), oconv(sl));
+    const char * sl = aspell_language_to_soundslike(lang, word.str(), word.size());
+    printf("%s\t%s\n", word.str(), sl);
   }
+  delete_aspell_language(lang);
 }
 
 //////////////////////////
@@ -1617,29 +1632,22 @@ void soundslike() {
 // munch
 //
 
+int put_word_munch(void *, const AspellWord * w)
+{
+  COUT << ' ' << w->str;
+  return true;
+}
+
 void munch() 
 {
-  using namespace aspeller;
-  CachePtr<Language> lang;
-  find_language(*options);
-  PosibErr<Language *> res = new_language(*options);
-  if (res.has_err()) {print_error(res.get_err()->mesg); exit(1);}
-  lang.reset(res.data);
-  Conv iconv(setup_conv(options, lang));
-  Conv oconv(setup_conv(lang, options));
+  AspellLanguage * lang = new_language();
   String word;
-  GuessInfo gi;
   while (CIN.getline(word)) {
-    lang->munch(iconv(word), &gi);
     COUT << word;
-    for (const aspeller::CheckInfo * ci = gi.head; ci; ci = ci->next)
-    {
-      COUT << ' ' << oconv(ci->word) << '/';
-      if (ci->pre_flag != 0) COUT << oconv(static_cast<char>(ci->pre_flag));
-      if (ci->suf_flag != 0) COUT << oconv(static_cast<char>(ci->suf_flag));
-    }
+    aspell_language_munch(lang, word.str(), word.size(), put_word_munch, 0);
     COUT << '\n';
   }
+  delete_aspell_language(lang);
 }
 
 //////////////////////////
@@ -1647,6 +1655,22 @@ void munch()
 // expand
 //
 
+int put_word_expand_1(void * d, const AspellWord * w)
+{
+  bool & prev = *static_cast<bool *>(d);
+  if (prev) COUT << ' ';
+  COUT << w->str;
+  prev = true;
+  return true;
+}
+
+int put_word_expand_2(void * d, const AspellWord * w)
+{
+  const char * str = (const char *)(d);
+  COUT << str << ' ' << w->str << '\n';
+  return true;
+}
+
 void expand() 
 {
   int level = 1;
@@ -1655,59 +1679,23 @@ void expand()
   int limit = INT_MAX;
   if (args.size() > 1)
     limit = atoi(args[1].c_str());
-  
-  using namespace aspeller;
-  CachePtr<Language> lang;
-  find_language(*options);
-  PosibErr<Language *> res = new_language(*options);
-  if (res.has_err()) {print_error(res.get_err()->mesg); exit(1);}
-  lang.reset(res.data);
-  Conv iconv(setup_conv(options, lang));
-  Conv oconv(setup_conv(lang, options));
-  String word, buf;
-  ObjStack exp_buf;
-  WordAff * exp_list;
+  AspellLanguage * lang = new_language();
+  String word;
   while (CIN.getline(word)) {
-    buf = word;
-    char * w = iconv(buf.mstr(), buf.size());
-    char * af = strchr(w, '/');
-    size_t s;
-    if (af != 0) {
-      s = af - w;
-      *af++ = '\0';
-    } else {
-      s = strlen(w);
-      af = w + s;
-    }
-    exp_buf.reset();
-    exp_list = lang->expand(w, af, exp_buf, limit);
     if (level <= 2) {
-      if (level == 2) 
-        COUT << word << ' ';
-      WordAff * p = exp_list;
-      while (p) {
-        COUT << oconv(p->word);
-        if (limit < INT_MAX && p->aff[0]) COUT << '/' << oconv((const char *)p->aff);
-        p = p->next;
-        if (p) COUT << ' ';
-      }
+      if (level == 2) COUT << word << ' ';
+      bool prev = false;
+      aspell_language_expand(lang, word.str(), word.size(), 
+                             put_word_expand_1, &prev, 
+                             limit);
       COUT << '\n';
     } else if (level >= 3) {
-      double ratio = 0;
-      if (level >= 4) {
-        for (WordAff * p = exp_list; p; p = p->next)
-          ratio += p->word.size;
-        ratio /= exp_list->word.size; // it is assumed the first
-                                      // expansion is just the root
-      }
-      for (WordAff * p = exp_list; p; p = p->next) {
-        COUT << word << ' ' << oconv(p->word);
-        if (limit < INT_MAX && p->aff[0]) COUT << '/' << oconv((const char *)p->aff);
-        if (level >= 4) COUT.printf(" %f\n", ratio);
-        else COUT << '\n';
-      }
+      aspell_language_expand(lang, word.str(), word.size(), 
+                             put_word_expand_2, const_cast<char *>(word.str()), 
+                             limit);
     }
   }
+  delete_aspell_language(lang);
 }
 
 //////////////////////////
@@ -1733,19 +1721,19 @@ static void print_wordaff(const String & base, const String & affs, Conv & oconv
     COUT.printf("/%s\n", oconv(affs));
 }
 
-static bool lower_equal(aspeller::Language * l, ParmString a, ParmString b)
+static bool lower_equal(sp::LangImpl * l, ParmString a, ParmString b)
 {
   if (a.size() != b.size()) return false;
   if (l->to_lower(a[0]) != l->to_lower(b[0])) return false;
   return memcmp(a + 1, b + 1, a.size() - 1) == 0;
 }
 
-void combine() 
+void combine()
 {
-  using namespace aspeller;
-  CachePtr<Language> lang;
+  using namespace sp;
+  CachePtr<LangImpl> lang;
   find_language(*options);
-  PosibErr<Language *> res = new_language(*options);
+  PosibErr<LangImpl *> res = new_lang_impl(*options);
   if (res.has_err()) {print_error(res.get_err()->mesg); exit(1);}
   lang.reset(res.data);
   Conv iconv(setup_conv(options, lang));
@@ -1785,6 +1773,52 @@ void combine()
   print_wordaff(base, affs, oconv);
 }
 
+//////////////////////////
+//
+// munch list
+//
+
+int get_word_ml(void * d, AspellWord * w) 
+{
+  String * str = static_cast<String *>(d);
+  bool res = CIN.getline(*str);
+  if (!res) return false;
+  w->str = str->str();
+  w->len = str->size();
+  return true;
+}
+
+int put_word_ml(void *, const AspellWord * w)
+{
+  COUT << w->str << '\n';
+  return true;
+}
+
+void munch_list()
+{
+  String word;
+  AspellMunchListParms * parms = new_aspell_munch_list_parms();
+
+  for (unsigned i = 0; i < args.size(); ++i) {
+    if (args[i] == "simple")      ; // FIXME: Warning
+    else if (args[i] == "single") parms->multi = false;
+    else if (args[i] == "multi")  parms->multi = true;
+    else if (args[i] == "keep")   parms->simplify = false;
+    else 
+    {
+      print_error(_("\"%s\" is not a valid flag for the \"munch-list\" command."),
+                  args[i]);
+      exit(1);
+    }
+  }
+
+  AspellLanguage * lang = new_language();
+  aspell_munch_list(lang, get_word_ml, &word, put_word_ml, 0, parms);
+  delete_aspell_munch_list_parms(parms);
+  delete_aspell_language(lang);
+
+}
+
 //////////////////////////
 //
 // dump affix
@@ -1793,7 +1827,7 @@ void combine()
 void dump_affix()
 {
   FStream in;
-  EXIT_ON_ERR(aspeller::open_affix_file(*options, in));
+  EXIT_ON_ERR(aspell::sp::open_affix_file(*options, in));
   
   String line;
   while (in.getline(line))
@@ -1836,7 +1870,7 @@ void print_help_line(char abrv, char dont_abrv, const char * name,
   printf("  %-27s %s\n", command.c_str(), tdesc); // FIXME: consider word wrapping
 }
 
-namespace acommon {
+namespace aspell {
   PosibErr<ConfigModule *> get_dynamic_filter(Config * config, ParmStr value);
 }
 
@@ -1876,14 +1910,16 @@ static const char * help_text[] =
   N_("  clean [strict]   cleans a word list so that every line is a valid word"),
   //N_("  filter           passes standard input through filters"),
   N_("  -v|version       prints a version line"),
+  N_("  munch-list [simple] [single|multi] [keep]"),
+  N_("    reduce the size of a word list via affix compression"),
   N_("  conv <from> <to> [<norm-form>]"),
   N_("    converts from one encoding to another"),
   N_("  norm (<norm-map> | <from> <norm-map> <to>) [<norm-form>]"),
   N_("    perform Unicode normalization"),
   usage_text[8],
   usage_text[9],
-  N_("  dump|create|merge master|personal|repl [word list]"),
-  N_("    dumps, creates or merges a master, personal, or replacement word list."),
+  N_("  dump|create|merge master|personal|repl [<name>]"),
+  N_("    dumps, creates or merges a master, personal, or replacement dictionary."),
   "",
   /* TRANSLATORS: "none", "internal" and "strict" are literal values
      and should not be translated. */
@@ -1898,7 +1934,7 @@ void print_help (bool verbose) {
   load_all_filters(options);
   if (verbose) {
     printf(_("\n"
-             "Aspell %s alpha.  Copyright 2000-2004 by Kevin Atkinson.\n"
+             "Aspell %s.  Copyright 2000-2004 by Kevin Atkinson.\n"
              "\n"), VERSION);
     for (unsigned i = 0; i < help_text_size; ++i)
       puts(gt_(help_text[i]));
